@@ -26,15 +26,24 @@
 #endif
 #include "aiounicast_select.hh"
 
+// additional headers
+#include <cassert>
+#include <sys/select.h>
+#include <unistd.h>
+#include <errno.h>
+#include <string.h>
+#include "mpz_srandom.hh"
+
 aiounicast_select::aiounicast_select
-	(const size_t n_in, const size_t j_in,
-	const std::vector<int> &fd_in_in,
-	const std::vector<int> &fd_out_in,
-	const std::vector<std::string> &key_in,
-	const size_t aio_default_scheduler_in,
-	const time_t aio_default_timeout_in,
-	const bool aio_is_authenticated_in,
-	const bool aio_is_encrypted_in):
+	(const size_t n_in,
+	 const size_t j_in,
+	 const std::vector<int> &fd_in_in,
+	 const std::vector<int> &fd_out_in,
+	 const std::vector<std::string> &key_in,
+	 const size_t aio_default_scheduler_in,
+	 const time_t aio_default_timeout_in,
+	 const bool aio_is_authenticated_in,
+	 const bool aio_is_encrypted_in):
 		aiounicast(n_in, j_in, aio_default_scheduler_in, aio_default_timeout_in,
 			aio_is_authenticated_in, aio_is_encrypted_in)
 {
@@ -66,7 +75,7 @@ aiounicast_select::aiounicast_select
 		if (maclen == 0)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_mac_get_algo_maclen() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_get_algo_maclen() failed" << std::endl;
 		}
 	}
 	else
@@ -76,13 +85,14 @@ aiounicast_select::aiounicast_select
 		unsigned char salt[maclen];
 		unsigned char key[maclen];
 		gcry_error_t err;
-		gcry_mac_hd_t *mac_in_hd = new gcry_mac_hd_t(), *mac_out_hd = new gcry_mac_hd_t();
+		gcry_mac_hd_t *mac_in_hd = new gcry_mac_hd_t();
+		gcry_mac_hd_t *mac_out_hd = new gcry_mac_hd_t();
 		mac_in.push_back(mac_in_hd), mac_out.push_back(mac_out_hd);
 		err = gcry_mac_open(mac_in[i], TMCG_GCRY_MAC_ALGO, 0, NULL); 				
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_mac_open() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_open() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		memset(salt, 0, sizeof(salt));
@@ -91,28 +101,28 @@ aiounicast_select::aiounicast_select
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_kdf_derive() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_kdf_derive() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		err = gcry_mac_setkey(*mac_in[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_mac_setkey() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_setkey() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		err = gcry_mac_open(mac_out[i], TMCG_GCRY_MAC_ALGO, 0, NULL); 				
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_mac_open() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_open() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		err = gcry_mac_setkey(*mac_out[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_mac_setkey() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_setkey() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 	}
@@ -124,13 +134,13 @@ aiounicast_select::aiounicast_select
 		if (keylen == 0)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_cipher_get_algo_keylen() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_get_algo_keylen() failed" << std::endl;
 		}
 		blklen = gcry_cipher_get_algo_blklen(TMCG_GCRY_ENC_ALGO);
 		if (blklen == 0)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_cipher_get_algo_blklen() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_get_algo_blklen() failed" << std::endl;
 		}
 	}
 	else
@@ -141,13 +151,14 @@ aiounicast_select::aiounicast_select
 		unsigned char key[keylen];
 		unsigned char iv[blklen];
 		gcry_error_t err;
-		gcry_cipher_hd_t *enc_in_hd = new gcry_cipher_hd_t(), *enc_out_hd = new gcry_cipher_hd_t();
+		gcry_cipher_hd_t *enc_in_hd = new gcry_cipher_hd_t();
+		gcry_cipher_hd_t *enc_out_hd = new gcry_cipher_hd_t();
 		enc_in.push_back(enc_in_hd), enc_out.push_back(enc_out_hd);
 		err = gcry_cipher_open(enc_in[i], TMCG_GCRY_ENC_ALGO, GCRY_CIPHER_MODE_CFB, 0); 				
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_cipher_open() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_open() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		memset(salt, 1, sizeof(salt)); // use a different salt to derive encryption key
@@ -156,14 +167,14 @@ aiounicast_select::aiounicast_select
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_kdf_derive() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_kdf_derive() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		err = gcry_cipher_setkey(*enc_in[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_cipher_setkey() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_setkey() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		iv_flag_in.push_back(false); // flag means: IV not yet received
@@ -171,14 +182,14 @@ aiounicast_select::aiounicast_select
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_cipher_open() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_open() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		err = gcry_cipher_setkey(*enc_out[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_cipher_setkey() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_setkey() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		gcry_create_nonce(iv, blklen); // unpredictable IV is sufficient
@@ -186,7 +197,7 @@ aiounicast_select::aiounicast_select
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "libgcrypt: gcry_cipher_setiv() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_setiv() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 		}
 		iv_flag_out.push_back(false); // flag means: IV not yet sent
@@ -197,8 +208,9 @@ aiounicast_select::aiounicast_select
 }
 
 bool aiounicast_select::Send
-	(mpz_srcptr m, const size_t i_in,
-	time_t timeout)
+	(mpz_srcptr m,
+	 const size_t i_in,
+	 time_t timeout)
 {
 	if (!aio_is_initialized)
 		return false;
@@ -252,7 +264,7 @@ bool aiounicast_select::Send
 		err = gcry_cipher_encrypt(*enc_out[i_in], buf + 1, realsize - 1, NULL, 0);
 		if (err)
 		{
-			std::cerr << "libgcrypt: gcry_cipher_encrypt() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_encrypt() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 			delete [] buf;
 			return false;
@@ -348,7 +360,7 @@ bool aiounicast_select::Send
 		err = gcry_mac_write(*mac_out[i_in], buf, realsize);
 		if (err)
 		{
-			std::cerr << "libgcrypt: gcry_mac_write() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_write() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 			delete [] buf;
 			return false;
@@ -417,7 +429,7 @@ bool aiounicast_select::Send
 		err = gcry_mac_read(*mac_out[i_in], macbuf, &macbuflen);
 		if (err)
 		{
-			std::cerr << "libgcrypt: gcry_mac_read() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_read() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 			delete [] macbuf;
 			return false;
@@ -425,7 +437,7 @@ bool aiounicast_select::Send
 		err = gcry_mac_reset(*mac_out[i_in]);
 		if (err)
 		{
-			std::cerr << "libgcrypt: gcry_mac_reset() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_reset() failed" << std::endl;
 			std::cerr << gcry_strerror(err) << std::endl;
 			delete [] macbuf;
 			return false;
@@ -490,8 +502,9 @@ bool aiounicast_select::Send
 }
 
 bool aiounicast_select::Send
-	(const std::vector<mpz_srcptr> &m, const size_t i_in,
-	time_t timeout)
+	(const std::vector<mpz_srcptr> &m,
+	 const size_t i_in,
+	 time_t timeout)
 {
 	if (!aio_is_initialized)
 		return false;
@@ -506,8 +519,10 @@ bool aiounicast_select::Send
 }
 
 bool aiounicast_select::Receive
-	(mpz_ptr m, size_t &i_out,
-	size_t scheduler, time_t timeout)
+	(mpz_ptr m,
+	 size_t &i_out,
+	 size_t scheduler,
+	 time_t timeout)
 {
 	if (!aio_is_initialized)
 		return false;
@@ -580,7 +595,7 @@ bool aiounicast_select::Receive
 						err = gcry_mac_write(*mac_in[i_out], tmp, newline_ptr);
 						if (err)
 						{
-							std::cerr << "libgcrypt: gcry_mac_write() failed" << std::endl;
+							std::cerr << "aiounicast_select: gcry_mac_write() failed" << std::endl;
 							std::cerr << gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
@@ -590,7 +605,7 @@ bool aiounicast_select::Receive
 						err = gcry_mac_write(*mac_in[i_out], &delim, 1);
 						if (err)
 						{
-							std::cerr << "libgcrypt: gcry_mac_write() failed" << std::endl;
+							std::cerr << "aiounicast_select: gcry_mac_write() failed" << std::endl;
 							std::cerr << gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
@@ -599,7 +614,7 @@ bool aiounicast_select::Receive
 						err = gcry_mac_verify(*mac_in[i_out], mac, maclen);
 						if (err)
 						{
-							std::cerr << "libgcrypt: gcry_mac_verify() for " << j << " from " << i_out << " failed" << std::endl;
+							std::cerr << "aiounicast_select: gcry_mac_verify() for " << j << " from " << i_out << " failed" << std::endl;
 							std::cerr << gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
@@ -607,7 +622,7 @@ bool aiounicast_select::Receive
 						err = gcry_mac_reset(*mac_in[i_out]);
 						if (err)
 						{
-							std::cerr << "libgcrypt: gcry_mac_reset() failed" << std::endl;
+							std::cerr << "aiounicast_select: gcry_mac_reset() failed" << std::endl;
 							std::cerr << gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
@@ -620,7 +635,7 @@ bool aiounicast_select::Receive
 						mpz_init(encval);
 						if (mpz_set_str(encval, tmp, TMCG_MPZ_IO_BASE) < 0)
 						{
-							std::cerr << "libgmp: mpz_set_str() for encval failed" << std::endl;
+							std::cerr << "aiounicast_select: mpz_set_str() for encval failed" << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
@@ -629,7 +644,7 @@ bool aiounicast_select::Receive
 						mpz_export(tmp, &realsize, 1, 1, 1, 0, encval);
 						if (realsize == 0)
 						{
-							std::cerr << "libgmp: mpz_export() failed for " << encval << std::endl;
+							std::cerr << "aiounicast_select: mpz_export() failed for " << encval << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
@@ -644,7 +659,7 @@ bool aiounicast_select::Receive
 						err = gcry_cipher_decrypt(*enc_in[i_out], tmp + 1, realsize - 1, NULL, 0);
 						if (err)
 						{
-							std::cerr << "libgcrypt: gcry_cipher_decrypt() failed" << std::endl;
+							std::cerr << "aiounicast_select: gcry_cipher_decrypt() failed" << std::endl;
 							std::cerr << gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
@@ -656,7 +671,7 @@ bool aiounicast_select::Receive
 					// extract value of m
 					if (mpz_set_str(m, tmp, TMCG_MPZ_IO_BASE) < 0)
 					{
-						std::cerr << "libgmp: mpz_set_str() for m from " << i_out << " failed" << std::endl;
+						std::cerr << "aiounicast_select: mpz_set_str() for m from " << i_out << " failed" << std::endl;
 						delete [] tmp, delete [] mac;
 						return false;
 					}
@@ -721,7 +736,7 @@ bool aiounicast_select::Receive
 							if (err)
 							{
 								aio_is_initialized = false;
-								std::cerr << "libgcrypt: gcry_cipher_setiv() failed" << std::endl;
+								std::cerr << "aiounicast_select: gcry_cipher_setiv() failed" << std::endl;
 								std::cerr << gcry_strerror(err) << std::endl;
 							}
 							iv_flag_in[i_out] = true; // IV is set
@@ -752,8 +767,10 @@ bool aiounicast_select::Receive
 }
 
 bool aiounicast_select::Receive
-	(std::vector<mpz_ptr> &m, size_t &i_out,
-	size_t scheduler, time_t timeout)
+	(std::vector<mpz_ptr> &m,
+	 size_t &i_out,
+	 size_t scheduler,
+	 time_t timeout)
 {
 	if (!aio_is_initialized)
 		return false;
