@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
- Copyright (C) 2005, 2006  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2005, 2006, 2009, 2015, 2016  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,14 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
+// include headers
+#ifdef HAVE_CONFIG_H
+	#include "libTMCG_config.h"
+#endif
+#include <libTMCG.hh>
+
+#ifdef FORKING
+
 #include <sstream>
 #include <cassert>
 #include <unistd.h>
@@ -25,7 +33,6 @@
 #include <sys/wait.h>
 
 #include "test_helper.h"
-#include <libTMCG.hh>
 #include "StiglicMPC.hh"
 #include "pipestream.hh"
 
@@ -64,7 +71,7 @@ int main
 	if (pid == 0)
 	{
 		/* BEGIN child code: participant B */
-		mpc = new StiglicMPC(16, list, 1);
+		mpc = new StiglicMPC(64, list, 1);
 		
 		MPC_Bit a, b, c, result;
 		bool x = false, y = false, z = false;
@@ -96,6 +103,14 @@ int main
 		mpc->MPC_OpenBitCommitment(result, x);
 		mpc->MPC_ComputeNEG(result, b);
 		mpc->MPC_OpenBitCommitment(result, x);
+		mpc->MPC_ComputeAND(result, a, a, false);
+		mpc->MPC_OpenBitCommitment(result, x);
+		mpc->MPC_ComputeAND(result, a, b, false);
+		mpc->MPC_OpenBitCommitment(result, x);
+		mpc->MPC_ComputeAND(result, b, a, false);
+		mpc->MPC_OpenBitCommitment(result, x);
+		mpc->MPC_ComputeAND(result, b, b, false);
+		mpc->MPC_OpenBitCommitment(result, x);
 		mpc->MPC_ComputeAND(result, a, a);
 		mpc->MPC_OpenBitCommitment(result, x);
 		mpc->MPC_ComputeAND(result, a, b);
@@ -129,7 +144,7 @@ int main
 	{
 		std::cout << "fork() = " << pid << std::endl;
 		/* Participant A */
-		mpc = new StiglicMPC(16, list, 0);
+		mpc = new StiglicMPC(64, list, 0);
 		
 		MPC_Bit a, b, c, result;
 		bool x = false, y = false, z = false;
@@ -184,7 +199,25 @@ int main
 		assert(mpc->MPC_OpenBitCommitment(result, x));
 		assert(x == true);
 		
+		std::cout << "ComputeAND (without VRHE)" << std::endl;
+		start_clock();
+		assert(mpc->MPC_ComputeAND(result, a, a, false));
+		assert(mpc->MPC_OpenBitCommitment(result, x));
+		assert(x == true);
+		assert(mpc->MPC_ComputeAND(result, a, b, false));
+		assert(mpc->MPC_OpenBitCommitment(result, x));
+		assert(x == false);
+		assert(mpc->MPC_ComputeAND(result, b, a, false));
+		assert(mpc->MPC_OpenBitCommitment(result, x));
+		assert(x == false);
+		assert(mpc->MPC_ComputeAND(result, b, b, false));
+		assert(mpc->MPC_OpenBitCommitment(result, x));
+		assert(x == false);
+		stop_clock();
+		std::cout << elapsed_time() << std::endl;
+
 		std::cout << "ComputeAND" << std::endl;
+		start_clock();
 		assert(mpc->MPC_ComputeAND(result, a, a));
 		assert(mpc->MPC_OpenBitCommitment(result, x));
 		assert(x == true);
@@ -197,6 +230,8 @@ int main
 		assert(mpc->MPC_ComputeAND(result, b, b));
 		assert(mpc->MPC_OpenBitCommitment(result, x));
 		assert(x == false);
+		stop_clock();
+		std::cout << elapsed_time() << std::endl;
 		
 		std::cout << "ComputeOR" << std::endl;
 		assert(mpc->MPC_ComputeOR(result, a, a));
@@ -230,12 +265,32 @@ int main
 	}
 	
 	// finalize
-	if (waitpid(pid, NULL, 0) != pid)
+	int wstatus = 0;
+	std::cerr << "waitpid(" << pid << ")" << std::endl;
+	if (waitpid(pid, &wstatus, 0) != pid)
 		perror("t-mpc (waitpid)");
-	
+	if (!WIFEXITED(wstatus))
+	{
+		std::cerr << "ERROR: ";
+		if (WIFSIGNALED(wstatus))
+			std::cerr << pid << " terminated by signal " << WTERMSIG(wstatus) << std::endl;
+		if (WCOREDUMP(wstatus))
+			std::cerr << pid << " dumped core" << std::endl;
+	}
 	list.clear();
 	delete A, delete B;
 	
 	delete pipe_in_A, delete pipe_out_A, delete pipe_in_B, delete pipe_out_B;
 	close(pipe1fd[0]), close(pipe1fd[1]), close(pipe2fd[0]), close(pipe2fd[1]);
 }
+
+#else
+
+int main
+	(int argc, char **argv)
+{
+	std::cout << "test skipped" << std::endl;
+	return 77;
+}
+
+#endif
