@@ -40,19 +40,26 @@ int main
 {
 	gcry_error_t ret;
 	tmcg_openpgp_octets_t in, out;
+	tmcg_openpgp_secure_octets_t ins, outs;
+
+	assert(init_libTMCG(true)); // enable libgcrypt's secure memory
 
 	// testing OctetsCompare(), OctetsCompareConstantTime(), OctetsCompareZero()
 	std::cout << "OctetsCompareZero() ";
 	do
 	{
-		in.clear(), out.clear();
+		in.clear(), out.clear(), ins.clear(), outs.clear();
 		for (size_t j = 0; j < 6; j++)
 			in.push_back(tmcg_mpz_wrandom_ui() % 2);
 		for (size_t j = 0; j < 6; j++)
 			out.push_back(tmcg_mpz_wrandom_ui() % 2);
+		for (size_t j = 0; j < in.size(); j++)
+			ins.push_back(in[j]);
+		for (size_t j = 0; j < out.size(); j++)
+			outs.push_back(out[j]);
 		assert((CallasDonnerhackeFinneyShawThayerRFC4880::
 			OctetsCompare(in, out) == CallasDonnerhackeFinneyShawThayerRFC4880::
-			OctetsCompareConstantTime(out, in)));
+			OctetsCompareConstantTime(outs, ins)));
 		std::cout << "~";
 	}
 	while (!CallasDonnerhackeFinneyShawThayerRFC4880::OctetsCompareZero(in));
@@ -126,7 +133,7 @@ int main
 	tmcg_openpgp_byte_t octcnt = 1;
 	size_t hashcnt = (16 + (octcnt & 15)) << ((octcnt >> 4) + 6);
 	size_t keylen = gcry_cipher_get_algo_keylen(TMCG_GCRY_ENC_ALGO);
-	std::string keystr = "Test";
+	tmcg_openpgp_secure_string_t keystr = "Test";
 	char salt[8];
 	char key[keylen];
 	gcry_error_t err;
@@ -137,17 +144,18 @@ int main
 		GCRY_KDF_ITERSALTED_S2K, TMCG_GCRY_MD_ALGO, salt, sizeof(salt),
 		hashcnt, sizeof(key), key);
 	assert(!err);
-	tmcg_openpgp_octets_t salt2, out2;
-	out.clear();
+	tmcg_openpgp_octets_t salt2;
+	tmcg_openpgp_secure_octets_t outs2;
+	outs.clear();
 	for (size_t i = 0; i < sizeof(key); i++)
-		out.push_back(key[i]); // copy the result
+		outs.push_back(key[i]); // copy the result
 	for (size_t i = 0; i < sizeof(salt); i++)
 		salt2.push_back(salt[i]); // copy the salt
 	std::cout << "S2KCompute(...)" << std::endl;
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		S2KCompute(TMCG_OPENPGP_HASHALGO_SHA256, keylen, keystr, salt2, true,
-		octcnt, out2);
-	assert(CallasDonnerhackeFinneyShawThayerRFC4880::OctetsCompare(out, out2));
+		octcnt, outs2);
+	assert(CallasDonnerhackeFinneyShawThayerRFC4880::OctetsCompareConstantTime(outs, outs2));
 
 	// create different asymmetric keys by using libgcrypt
 	size_t erroff = 0;
@@ -165,6 +173,13 @@ int main
 	std::cout << "gcry_pk_genkey(..., [dsa])" << std::endl;
 	ret = gcry_pk_genkey(&dsakey, dsaparms);
 	assert(!ret);
+	gcry_sexp_t ecdsakey, ecdsaparms;
+	std::cout << "gcry_sexp_build(...)" << std::endl;
+	ret = gcry_sexp_build(&ecdsaparms, &erroff, "(genkey (ecdsa (curve secp384r1)))");
+	assert(!ret);
+	std::cout << "gcry_pk_genkey(..., [ecdsa])" << std::endl;
+	ret = gcry_pk_genkey(&ecdsakey, ecdsaparms);
+	assert(!ret);
 	gcry_sexp_t dsakey2, dsaparms2;
 	std::cout << "gcry_sexp_build(...)" << std::endl;
 	ret = gcry_sexp_build(&dsaparms2, &erroff, "(genkey (dsa (nbits 4:2048)))");
@@ -183,7 +198,8 @@ int main
 
 	// testing SymmetricEncryptAES256() and SymmetricDecryptAES256()
 	// testing AsymmetricEncryptElgamal() and AsymmetricDecryptElgamal()
-	tmcg_openpgp_octets_t lit, seskey, prefix, enc, subkeyid;
+	tmcg_openpgp_octets_t lit, prefix, enc, subkeyid;
+	tmcg_openpgp_secure_octets_t seskey;
 	std::string m = "This is a test message.", armored_message;
 	in.clear();
 	for (size_t i = 0; i < m.length(); i++)
@@ -293,6 +309,23 @@ int main
 	gcry_mpi_release(r);
 	gcry_mpi_release(s);
 
+	// testing AsymmetricSignECDSA() and AsymmetricVerifyECDSA()
+	hash.clear(), trailer.clear(), left.clear(), sig.clear();
+	r = gcry_mpi_new(1024);
+	s = gcry_mpi_new(1024);
+	std::cout << "AsymmetricSignECDSA(...)" << std::endl;
+	ret = CallasDonnerhackeFinneyShawThayerRFC4880::AsymmetricSignECDSA(hash,
+		ecdsakey, r, s);
+	assert(!ret);
+	std::cout << "r = " << r << std::endl;
+	std::cout << "s = " << s << std::endl;
+	std::cout << "AsymmetricVerifyECDSA(...)" << std::endl;
+	ret = CallasDonnerhackeFinneyShawThayerRFC4880::AsymmetricVerifyECDSA(hash,
+		ecdsakey, r, s);
+	assert(!ret);
+	gcry_mpi_release(r);
+	gcry_mpi_release(s);
+
 	// testing AsymmetricSignRSA() and AsymmetricVerifyRSA()
 	hash.clear(), trailer.clear(), left.clear(), sig.clear();
 	for (size_t i = 0; i < 2; i++)
@@ -328,20 +361,21 @@ int main
 		ArmorEncode(TMCG_OPENPGP_ARMOR_MESSAGE, out, armored_message);
 	std::cout << armored_message << std::endl;
 	std::cout << "AsymmetricDecryptRSA(...)" << std::endl;
-	tmcg_openpgp_octets_t seskey2;
+	tmcg_openpgp_secure_octets_t seskey2;
 	ret = CallasDonnerhackeFinneyShawThayerRFC4880::AsymmetricDecryptRSA(me,
 		rsakey, seskey2);
 	assert(!ret);
-	assert(CallasDonnerhackeFinneyShawThayerRFC4880::OctetsCompare(seskey,
+	assert(CallasDonnerhackeFinneyShawThayerRFC4880::OctetsCompareConstantTime(seskey,
 		seskey2));
 	gcry_mpi_release(me);
 
 	// testing PublicKeyBlockParse(), FingerprintCompute(), KeyidCompute()
 	tmcg_openpgp_octets_t all, pub, uid, uidsig, sub, subsig, pubflags, empty;
-	tmcg_openpgp_octets_t subflags, keyid, pub_hashing, uidsig_hashing;
+	tmcg_openpgp_octets_t subflags, keyid, pub_hashing, uidsig_hashing, issuer;
 	tmcg_openpgp_octets_t uidsig_left, sub_hashing, subsig_hashing, subsig_left;
 	tmcg_openpgp_octets_t sec, subsec;
-	std::string armored_pubkeyblock, passphrase = "FCK!NSA";
+	std::string armored_pubkeyblock, username = "Test";
+	tmcg_openpgp_secure_string_t passphrase = "FCK!NSA";
 	gcry_mpi_t p, q, g, y, x;
 	std::cout << "gcry_sexp_extract_param(...)" << std::endl;
 	ret = gcry_sexp_extract_param(dsakey, NULL, "pqgy", &p, &q, &g, &y, NULL);
@@ -353,10 +387,10 @@ int main
 		TMCG_OPENPGP_PKALGO_DSA, p, q, g, y, pub);
 	CallasDonnerhackeFinneyShawThayerRFC4880::PacketSecEncode(creation,
 		TMCG_OPENPGP_PKALGO_DSA, p, q, g, y, x, passphrase, sec);
-	for (size_t i = 6; i < pub.size(); i++)
-		pub_hashing.push_back(pub[i]);
+	CallasDonnerhackeFinneyShawThayerRFC4880::PacketBodyExtract(pub, 3, pub_hashing);
 	CallasDonnerhackeFinneyShawThayerRFC4880::KeyidCompute(pub_hashing, keyid);
-	CallasDonnerhackeFinneyShawThayerRFC4880::PacketUidEncode(keystr, uid);
+	CallasDonnerhackeFinneyShawThayerRFC4880::FingerprintCompute(pub_hashing, issuer);
+	CallasDonnerhackeFinneyShawThayerRFC4880::PacketUidEncode(username, uid);
 	pubflags.push_back(0x01 | 0x02);  // certify other keys and sign data
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		PacketSigPrepareSelfSignature(TMCG_OPENPGP_SIGNATURE_POSITIVE_CERTIFICATION,
@@ -364,7 +398,7 @@ int main
 			uidsig_hashing); 
 	hash.clear();
 	CallasDonnerhackeFinneyShawThayerRFC4880::CertificationHash(pub_hashing,
-		keystr, empty, uidsig_hashing, TMCG_OPENPGP_HASHALGO_SHA256, hash,
+		username, empty, uidsig_hashing, TMCG_OPENPGP_HASHALGO_SHA256, hash,
 		uidsig_left);
 	r = gcry_mpi_new(2048);
 	s = gcry_mpi_new(2048);
@@ -388,10 +422,9 @@ int main
 	subflags.push_back(0x04 | 0x08); // encrypt communications and storage
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		PacketSigPrepareSelfSignature(TMCG_OPENPGP_SIGNATURE_SUBKEY_BINDING,
-			TMCG_OPENPGP_HASHALGO_SHA256, time(NULL), 1000, subflags, keyid,
+			TMCG_OPENPGP_HASHALGO_SHA256, time(NULL), 1000, subflags, issuer,
 			subsig_hashing);
-	for (size_t i = 6; i < sub.size(); i++)
-		sub_hashing.push_back(sub[i]);
+	CallasDonnerhackeFinneyShawThayerRFC4880::PacketBodyExtract(sub, 3, sub_hashing);
 	hash.clear();
 	CallasDonnerhackeFinneyShawThayerRFC4880::KeyHash(pub_hashing, sub_hashing,
 		subsig_hashing, TMCG_OPENPGP_HASHALGO_SHA256, hash, subsig_left);
@@ -431,8 +464,8 @@ int main
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		KeyidCompute(primary->pub_hashing, kid);
 	std::cout << kid << std::endl;
-	std::cout << "!primary->weak()" << std::endl;
-	bool check_ok = primary->weak(3);
+	std::cout << "!primary->Weak()" << std::endl;
+	bool check_ok = primary->Weak(3);
 	assert(!check_ok);
 	delete primary;
 	delete ring;
@@ -451,7 +484,7 @@ int main
 	parse_ok = CallasDonnerhackeFinneyShawThayerRFC4880::
 		SignatureParse(armored_signature, 3, signature);
 	assert(parse_ok);
-	assert(signature->good());
+	assert(signature->Good());
 	std::cout << "PrintInfo()" << std::endl;
 	signature->PrintInfo();
 	std::cout << "CheckValidity()" << std::endl;
@@ -481,18 +514,17 @@ int main
 	assert(!ret);
 	CallasDonnerhackeFinneyShawThayerRFC4880::PacketPubEncode(creation,
 		TMCG_OPENPGP_PKALGO_DSA, p, q, g, y, pub2);
-	for (size_t i = 6; i < pub2.size(); i++)
-		pub_hashing2.push_back(pub2[i]);
+	CallasDonnerhackeFinneyShawThayerRFC4880::PacketBodyExtract(pub2, 3, pub_hashing2);
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		KeyidCompute(pub_hashing2, keyid2);
-	CallasDonnerhackeFinneyShawThayerRFC4880::PacketUidEncode(keystr, uid2);
+	CallasDonnerhackeFinneyShawThayerRFC4880::PacketUidEncode(username, uid2);
 	CallasDonnerhackeFinneyShawThayerRFC4880::
 		PacketSigPrepareSelfSignature(TMCG_OPENPGP_SIGNATURE_POSITIVE_CERTIFICATION,
 			TMCG_OPENPGP_HASHALGO_SHA256, time(NULL), 1000, pubflags, keyid2,
 			uidsig_hashing2);
 	hash.clear();
 	CallasDonnerhackeFinneyShawThayerRFC4880::CertificationHash(pub_hashing2,
-		keystr, empty, uidsig_hashing2, TMCG_OPENPGP_HASHALGO_SHA256, hash,
+		username, empty, uidsig_hashing2, TMCG_OPENPGP_HASHALGO_SHA256, hash,
 		uidsig_left2);
 	r = gcry_mpi_new(2048);
 	s = gcry_mpi_new(2048);
@@ -508,8 +540,7 @@ int main
 		PacketSigPrepareSelfSignature(TMCG_OPENPGP_SIGNATURE_SUBKEY_BINDING,
 			TMCG_OPENPGP_HASHALGO_SHA256, time(NULL), 1000, subflags, keyid2,
 			subsig_hashing2);
-	for (size_t i = 6; i < sub2.size(); i++)
-		sub_hashing2.push_back(sub2[i]);
+	CallasDonnerhackeFinneyShawThayerRFC4880::PacketBodyExtract(sub2, 3, sub_hashing2);
 	hash.clear();
 	CallasDonnerhackeFinneyShawThayerRFC4880::KeyHash(pub_hashing2, sub_hashing2,
 		subsig_hashing2, TMCG_OPENPGP_HASHALGO_SHA256, hash, subsig_left2);
@@ -533,7 +564,11 @@ int main
 	parse_ok = CallasDonnerhackeFinneyShawThayerRFC4880::
 		PublicKeyringParse(armored_pubkeyringblock, 3, ring);
 	assert(parse_ok);
-	assert((ring->size() == 2));
+	assert((ring->Size() == 2));
+	assert((ring->List("") == 2));
+	assert((ring->Check(0) == 2));
+	ring->Reduce();
+	assert((ring->Size() == 2));
 	delete ring;
 	gcry_mpi_release(p);
 	gcry_mpi_release(q);
@@ -661,10 +696,44 @@ int main
 	gcry_sexp_release(elgkey);
 	gcry_sexp_release(dsaparms);
 	gcry_sexp_release(dsakey);
+	gcry_sexp_release(ecdsaparms);
+	gcry_sexp_release(ecdsakey);
 	gcry_sexp_release(dsaparms2);
 	gcry_sexp_release(dsakey2);
 	gcry_sexp_release(rsaparms);
 	gcry_sexp_release(rsakey);
+
+	// test externally generated public key
+	std::string mallory_armored =
+"-----BEGIN PGP PUBLIC KEY BLOCK-----\r\n\r\n"
+"mDMEW3P+yxYJKwYBBAHaRw8BAQdAFWO/NfdgP1sX27GUdaporGJkMhCNfr/ZlyMm\r\n"
+"6i8vwD20B01hbGxvcnmIkAQTFggAOBYhBOOe9Pt+rWYkOw5dGnchfMwJwVMrBQJb\r\n"
+"c/7LAhsDBQsJCAcCBhUICQoLAgQWAgMBAh4BAheAAAoJEHchfMwJwVMrrYYBAKEK\r\n"
+"Y0uKnES/YPmXB6Jj5qmK82MX+mJx8hccHPdsJb/nAQDusYVS/4Xx+uWK9/zntrDq\r\n"
+"Jf1hYsjSxe/03+cT2ze/A7g4BFtz/ssSCisGAQQBl1UBBQEBB0BaH+8/c76A3gPe\r\n"
+"a631zMYQou+bF7l9x25iPrWrivoCNgMBCAeIeAQYFggAIBYhBOOe9Pt+rWYkOw5d\r\n"
+"GnchfMwJwVMrBQJbc/7LAhsMAAoJEHchfMwJwVMrsVoA/09EgCBkINfruB0MomXK\r\n"
+"KiG6cUjGtAO0aURabyiKWiS1AP97KMY0ixHTQIV0sCR3LGIIu3ojnNtapLuqzqJt\r\n"
+"uO6+DQ==\r\n"
+"=VqXx\r\n"
+"-----END PGP PUBLIC KEY BLOCK-----\r\n";
+	TMCG_OpenPGP_Pubkey *mallory = NULL;
+	ring = new TMCG_OpenPGP_Keyring();
+	std::cout << "PublicKeyBlockParse(mallory_armored, 3, mallory)" << std::endl;
+	parse_ok = CallasDonnerhackeFinneyShawThayerRFC4880::
+		PublicKeyBlockParse(mallory_armored, 3, mallory);
+	assert(parse_ok);
+	std::cout << "CheckSelfSignatures()" << std::endl;
+	parse_ok = mallory->CheckSelfSignatures(ring, 3);
+	assert(parse_ok);
+	std::cout << "CheckSubkeys()" << std::endl;
+	parse_ok = mallory->CheckSubkeys(ring, 3);
+	assert(parse_ok);
+	std::cout << "!primary->Weak()" << std::endl;
+	check_ok = mallory->Weak(3);
+	assert(!check_ok);
+	delete mallory;
+	delete ring;
 	
 	return 0;
 }
